﻿namespace Battleship.MovementStrategy;

public abstract class AbstractStrategy
{
    /// <summary>
    /// Перемещаемый объект
    /// </summary>
    private IMoveableObjectcs? _moveableObjects;

    /// <summary>
    /// Статус перемещения
    /// </summary>
    private StrategyStatus _state = StrategyStatus.NotIInit;

    /// <summary>
    /// Ширина поля
    /// </summary>
    protected int FieldWidth {  get; private set; }

    /// <summary>
    /// Высота поля
    /// </summary>
    protected int FieldHeight { get; private set;}

    /// <summary>
    /// Статус перемещения
    /// </summary>
    /// <returns></returns>
    public StrategyStatus GetStatus() { return _state; }

    /// <summary>
    /// Установка данных
    /// </summary>
    /// <param name="moveableObject">Перемещаемый объект</param>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    public void SetData(IMoveableObjectcs moveableObjectcs, int width, int height)
    {
        if (moveableObjectcs == null)
        {
            _state = StrategyStatus.NotIInit;
            return;
        }

        _state = StrategyStatus.InProgress;
        _moveableObjects = moveableObjectcs;
        FieldWidth = width;
        FieldHeight = height;
    }

    /// <summary>
    /// Шаг перемещения
    /// </summary>
    public void MakeStep()
    {
        if (_state != StrategyStatus.InProgress)
        {
            return;
        }

        if (IsTargetDestination())
        {
            _state = StrategyStatus.Finish;
            return;
        }

        MoveToTarget();
    }

    /// <summary>
    /// Перемещение влево
    /// </summary>
    /// <returns></returns>
    protected bool MoveLeft() => MoveTo(MovementDirection.Left);

    /// <summary>
    /// Перемещение вправо
    /// </summary>
    /// <returns></returns>
    protected bool MoveRight() => MoveTo(MovementDirection.Right);

    /// <summary>
    /// Перемещение вверх
    /// </summary>
    /// <returns></returns>
    protected bool MoveUp() => MoveTo(MovementDirection.Up);

    /// <summary>
    /// Перемещение вниз
    /// </summary>
    /// <returns></returns>
    protected bool MoveDown() => MoveTo(MovementDirection.Down);

    /// <summary>
    /// Параметры объекта
    /// </summary>
    protected ObjectParameters? GetObjectParameters => _moveableObjects?.GetObjectPosition;

    /// <summary>
    /// Шаг объекта
    /// </summary>
    /// <returns></returns>
    protected int? GetStep()
    {
        if (_state != StrategyStatus.InProgress)
        {
            return null;
        }
        return _moveableObjects?.GetStep;
    }

    /// <summary>
    /// Перемещение к цели
    /// </summary>
    protected abstract void MoveToTarget();

    /// <summary>
    /// Достигнута ли цель
    /// </summary>
    /// <returns></returns>
    protected abstract bool IsTargetDestination();

    /// <summary>
    /// Попытка перемещения в требуемом направлении 
    /// </summary>
    /// <param name="movementDirection"></param>
    /// <returns></returns>
    private bool MoveTo(MovementDirection movementDirection)
    {
        if (_state != StrategyStatus.InProgress)
        {
            return false;
        }

        return _moveableObjects?.TryMoveObject(movementDirection) ?? false;
    }
}
